from random import shuffle

###################################################################################################
##########                         EXAMPLE: CARD AND DECK CLASSES                        ##########
###################################################################################################
##########                       EXERCISE: ADD METHODS TO CARD CLASS                     ##########
###################################################################################################
# Step 1: Create a method numerical_value that returns a number to represent each card’s rank. 
#         The method should take one arguments in addition to self: a bool aces_high.
#         Face cards should have values jack = 11, queen = 12, king = 13, ace = 1 or 14 
#         (depending on aces_high)
# Step 2: Create a method higher_rank that takes two arguments in addition to self (a Card other 
#         and a bool aces_high) and returns True if the self.rank is greater than other.rank
#         You may find it useful to call the numerical_value method
# Step 3: Come up with a few good test cases for each method and test the methods
###################################################################################################

class Card:
    """
    A class used to represent a playing card
    
    Attributes
    ----------
    suit (string): the suit of the card
    rank (string or int): the rank of the card
    """
    def __init__(self, suit, rank):
        """
        Initializes a Card object
        
        Arguments:
            suit (string): the suit of the card
            rank (string or int): the rank of the card
        """ 
        self.suit = suit
        self.rank = rank


class Deck:
    """
    A class used to represent a playing card
    
    Attributes
    ----------
    cards (List[card]): the cards in the deck
    
    Methods
    -------
    draw()
        Removes a card from the deck and returns it
    """
    def __init__(self):
        """
        Initializes a Deck object
        
        Arguments: None
        """ 
        self.cards = []
        for rank in [2, 3, 4, 5, 6, 7, 8, 9, 10, "J", "Q", "K", "A"]:
            for suit in ["Spades", "Clubs", "Hearts", "Diamonds"]:
                self.cards.append(Card(suit, rank))
        shuffle(self.cards)

    def draw(self):
        """
        Returns the card at index 0 from the deck and removes it
        
        Arguments: None

        Modifies:
            self.cards
             
        Returns:
            Card: the card at the top of the deck (index 0)
        """
        return self.cards.pop(0)

a_card = Card("Hearts", "J")
print(a_card.suit)
a_card.suit = "Diamonds"
print(a_card)
print(a_card == Card("Diamonds", "J"))


###################################################################################################
##########                             EXAMPLE: STUDENT CLASS                            ##########
###################################################################################################
##########               EXERCISE: ADD ATTRIBUTES AND METHODS TO CARD CLASS              ##########
###################################################################################################

class Student:
    """
    A class used to represent a student
    
    Attributes
    ----------
    first_name (string): the student's given name
    last_name (string): the student's family name
    
    Methods
    -------
    full_name()
        Returns the student's full name
    """
    def __init__(self, first_name, last_name):
        """
        Initializes a Student object
        
        Arguments:
            first_name (string): the student's given name
            last_name (string): the student's family name
        """
        self.first_name = first_name
        self.last_name = last_name

    def full_name(self):
        """
        Returns the student's full name
        
        Arguments: None
             
        Returns:
            string: the full name
        """
        return self.first_name + " " + self.last_name
